==> carryon-fabric-1.21.1-2.2.3.3-sources.jar: tschipp/carryon/common/pickupcondition/PickupCondition.java <==
/*
 * GNU Lesser General Public License v3
 * Copyright (C) 2024 Tschipp
 * mrtschipp@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package tschipp.carryon.common.pickupcondition;

import com.mojang.brigadier.exceptions.CommandSyntaxException;
import com.mojang.serialization.Codec;
import com.mojang.serialization.DataResult;
import tschipp.carryon.platform.Services;
import tschipp.carryon.utils.StringHelper;

import java.util.Map;
import java.util.Map.Entry;
import net.minecraft.class_1297;
import net.minecraft.class_2259;
import net.minecraft.class_2259.class_7211;
import net.minecraft.class_2680;
import net.minecraft.class_2769;
import net.minecraft.class_3222;
import net.minecraft.class_7923;

public class PickupCondition
{
	public static Codec<PickupCondition> CODEC = Codec.STRING.comapFlatMap(PickupCondition::of, pickupCondition -> pickupCondition.str);

	private String str, cond, match;
	private boolean wildcards;

	private PickupCondition(String str, String cond, String match)
	{
		this.str = str;
		this.cond = cond;
		this.match = match;
	}

	public static DataResult<PickupCondition> of(String str)
	{
		if(!(str.contains("(") && str.endsWith(")")))
			return DataResult.error(() -> "Error while parsing: "+ str +". Pickup Condition must contain proper brackets.");

		String cond = str.substring(str.indexOf("(") + 1, str.length()-1);

		String match = str.substring(0, str.indexOf("("));

		PickupCondition condition = new PickupCondition(str, cond, match);
		if(match.contains("*"))
			condition.wildcards = true;

		return DataResult.success(condition);
	}

	public boolean matches(class_2680 state)
	{
		if(wildcards)
		{
			String name = match.contains("[") ? match.substring(0, match.indexOf("[")) : match;
			String[] split = name.replace("*", ",").split(",");
			String stateName = class_7923.field_41175.method_10221(state.method_26204()).toString();

			if(StringHelper.matchesWildcards(stateName, split))
			{
				if(match.contains("["))
				{
					stateName = stateName + match.substring(match.indexOf("["));
					class_7211 result = parseState(stateName);
					return matchesProperties(state, result.comp_623());
				}
				else
					return true;
			}
			else
				return false;
		}
		else
		{
			class_7211 res = parseState(match);
			return res.comp_622().method_26204() == state.method_26204() && matchesProperties(state, res.comp_623());
		}
	}

	public boolean matches(class_1297 entity)
	{
		String entityName = class_7923.field_41177.method_10221(entity.method_5864()).toString();
		if(wildcards)
		{
			String[] split = match.replace("*", ",").split(",");
			return StringHelper.matchesWildcards(entityName, split);
		}
		else
			return entityName.equals(match);
	}

	public String getCondition()
	{
		return cond;
	}

	public boolean isFulfilled(class_3222 player)
	{
		return Services.GAMESTAGES.hasStage(player, cond);
	}

	private class_7211 parseState(String state)
	{
		try {
			class_7211 result = class_2259.method_41957(class_7923.field_41175.method_46771(), state, false);
			return result;
		} catch (CommandSyntaxException e) {
			e.printStackTrace();
		}
		return null;
	}

	private boolean matchesProperties(class_2680 state, Map<class_2769<?>, Comparable<?>> props)
	{
		for(var entry : props.entrySet())
		{
			var val = state.method_11654(entry.getKey());
			if(val != entry.getValue())
				return false;
		}
		return true;
	}


}

==> carryon-fabric-1.21.9-2.8.0-sources.jar: tschipp/carryon/common/pickupcondition/PickupCondition.java <==
/*
 * GNU Lesser General Public License v3
 * Copyright (C) 2024 Tschipp
 * mrtschipp@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package tschipp.carryon.common.pickupcondition;

import com.mojang.brigadier.exceptions.CommandSyntaxException;
import com.mojang.serialization.Codec;
import com.mojang.serialization.DataResult;
import tschipp.carryon.platform.Services;
import tschipp.carryon.utils.StringHelper;

import java.util.Map;
import net.minecraft.class_1297;
import net.minecraft.class_2259;
import net.minecraft.class_2259.class_7211;
import net.minecraft.class_2680;
import net.minecraft.class_2769;
import net.minecraft.class_3222;
import net.minecraft.class_7923;

public class PickupCondition
{
	public static Codec<PickupCondition> CODEC = Codec.STRING.comapFlatMap(PickupCondition::of, pickupCondition -> pickupCondition.str);

	private String str, cond, match;
	private boolean wildcards;

	private PickupCondition(String str, String cond, String match)
	{
		this.str = str;
		this.cond = cond;
		this.match = match;
	}

	public static DataResult<PickupCondition> of(String str)
	{
		if(!(str.contains("(") && str.endsWith(")")))
			return DataResult.error(() -> "Error while parsing: "+ str +". Pickup Condition must contain proper brackets.");

		String cond = str.substring(str.indexOf("(") + 1, str.length()-1);

		String match = str.substring(0, str.indexOf("("));

		PickupCondition condition = new PickupCondition(str, cond, match);
		if(match.contains("*"))
			condition.wildcards = true;

		return DataResult.success(condition);
	}

	public boolean matches(class_2680 state)
	{
		if(wildcards)
		{
			String name = match.contains("[") ? match.substring(0, match.indexOf("[")) : match;
			String[] split = name.replace("*", ",").split(",");
			String stateName = class_7923.field_41175.method_10221(state.method_26204()).toString();

			if(StringHelper.matchesWildcards(stateName, split))
			{
				if(match.contains("["))
				{
					stateName = stateName + match.substring(match.indexOf("["));
					class_7211 result = parseState(stateName);
					return matchesProperties(state, result.comp_623());
				}
				else
					return true;
			}
			else
				return false;
		}
		else
		{
			class_7211 res = parseState(match);
			return res.comp_622().method_26204() == state.method_26204() && matchesProperties(state, res.comp_623());
		}
	}

	public boolean matches(class_1297 entity)
	{
		String entityName = class_7923.field_41177.method_10221(entity.method_5864()).toString();
		if(wildcards)
		{
			String[] split = match.replace("*", ",").split(",");
			return StringHelper.matchesWildcards(entityName, split);
		}
		else
			return entityName.equals(match);
	}

	public String getCondition()
	{
		return cond;
	}

	public boolean isFulfilled(class_3222 player)
	{
		return Services.GAMESTAGES.hasStage(player, cond);
	}

	private class_7211 parseState(String state)
	{
		try {
			class_7211 result = class_2259.method_41957(class_7923.field_41175, state, false);
			return result;
		} catch (CommandSyntaxException e) {
			e.printStackTrace();
		}
		return null;
	}

	private boolean matchesProperties(class_2680 state, Map<class_2769<?>, Comparable<?>> props)
	{
		for(var entry : props.entrySet())
		{
			var val = state.method_11654(entry.getKey());
			if(val != entry.getValue())
				return false;
		}
		return true;
	}


}
